// 4 digit clock
// Vdd = 3V
// Display: LUMEX LCD-S401C39TF
// 2 * 22pF & 32kHz tuning fork crystal on T1OSO & T1OSI
// VLCD3/RB3 to Vdd
// RB5 - button increment minutes
// MCLR/RE3 - button hours/minute incrmenet - hold low to increment hours

// PIC16LF1907 Configuration Bit Settings

// 'C' source line config statements

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
//#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
//#pragma config CLKOUTEN = ON    // Clock Out Enable (CLKOUT function is enabled on the CLKOUT pin)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown-out Reset (Low-Power BOR is disabled)
//#pragma config LVP = ON         // Low-Voltage Programming Enable (Low-voltage programming enabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

#include <xc.h>

#include <stdint.h>
#include <stdbool.h>
#include "lcd.h"

//#define _XTAL_FREQ 31000
//#define _XTAL_FREQ 32768
#define _XTAL_FREQ 16000000

typedef struct
{
    uint8_t u8Hours10;
    uint8_t u8Hours1;
    uint8_t u8Mins10;
    uint8_t u8Mins1;
    uint8_t u8Secs;
} CLKSTRUCT;

static CLKSTRUCT _cs={1,3,3,8,57};

int main(void) 
{
    bool bDebounce=false; // RB5 debounce
    bool bButtonState=false; // RB5 button state
    bool bButtonStateLast=false; // RB5 previous button state
    
    LATA=0;
    LATB=0;
    LATC=0;
    LATD=0;
    LATE=0;
    
    ANSELA=0;
    ANSELB=0;
    ANSELE=0;
    
    TRISA=0;
    TRISB=0b00101000; // RB5 button input, RB3 VLCD3 tied to Vdd
//    TRISC=0b00000001; // RC0/T1CKI
    TRISC=0;
    TRISD=0;
    TRISE=0b0001000; // RE3 is MCLR or button input (hours/minutes)
    
    WPUB=0b00100000; // RB5 button input
    WPUE=0b00001000; // RE3/#MCLR
    OPTION_REGbits.nWPUEN=0; // Global enable pullups
    
//    OSCCONbits.IRCF=0b0000; // 0b000x = 31 kHz LF
    OSCCONbits.IRCF=0b1111; // 0b1111 = 16 MHz

    T1CONbits.TMR1CS=0b10; // External osc/xtal osc
    T1CONbits.T1OSCEN=1; // Enable T1OSC (Secondary oscillator)
    T1CONbits.nT1SYNC=1; // 1 => Not suync'd with Fcy  
    TMR1=0xF800;

    while (!OSCSTATbits.T1OSCR) // Wait for T1OSC to be ready
    {
        NOP();
    }
//    OSCCONbits.SCS=0b01; // 0b01 = Secondary oscillator
   
//    T1CONbits.T1CKPS=0b10; // 0b10 => 1:4 prescale
    T1CONbits.T1CKPS=0b11; // 0b11 => 1:8 prescale
    PIE1bits.TMR1IE=1;
    T1CONbits.TMR1ON=1;
    
    IOCBNbits.IOCBN5=1;
    IOCBPbits.IOCBP5=1;
    IOCBFbits.IOCBF5=0;
    INTCONbits.IOCIF=0;    
    INTCONbits.IOCIE=1;
    
    LCDInit();
    
    LATAbits.LATA2=1;
    LCD_DATA_COL=1;
    __delay_ms(250);
    LATAbits.LATA2=0;
    LCD_DATA_COL=0;
    __delay_ms(250);
    LATAbits.LATA2=1;
    LCD_DATA_COL=1;
    __delay_ms(250);
    LATAbits.LATA2=0;
    LCD_DATA_COL=0;
    __delay_ms(250);
    LATAbits.LATA2=1;
    LCD_DATA_COL=1;
    __delay_ms(1500);
    LATAbits.LATA2=0;    
    LCD_DATA_COL=0;
    
    INTCONbits.PEIE=1;
    INTCONbits.GIE=0;
    
    LCDDigit1Set(_cs.u8Hours10);
    LCDDigit2Set(_cs.u8Hours1);
    LCDDigit3Set(_cs.u8Mins10);
    LCDDigit4Set(_cs.u8Mins1);

    while (1)
    {
#ifndef NO_SLEEP
        SLEEP();
#else
        while (!INTCONbits.IOCIF && !PIR1bits.TMR1IF)
        {
            NOP();
        }
#endif

        if (PIR1bits.TMR1IF)
        {
            TMR1=0xF800;
            PIR1bits.TMR1IF=0;
            
            if (bDebounce)
            {
                bDebounce=false;
                INTCONbits.IOCIE=1; // Enable IOC again
                IOCBFbits.IOCBF5=0;
                INTCONbits.IOCIF=0;
                
                bButtonStateLast=bButtonState;
                bButtonState=PORTBbits.RB5==0;
                if (bButtonState && !bButtonStateLast) // if button down but last button state was up...
                {
                    if (PORTEbits.RE3) // Is the hour/minute button held down?
                    {
                        if (++_cs.u8Mins1>9)
                        {
                            _cs.u8Mins1=0;
                            if (++_cs.u8Mins10>5)
                            {
                                _cs.u8Mins10=0;
                            }
                            LCDDigit3Set(_cs.u8Mins10);
                        }
                        LCDDigit4Set(_cs.u8Mins1);
                    }
                    else
                    {
                        if (++_cs.u8Hours1>9)
                        {
                            _cs.u8Hours1=0;
                            if (++_cs.u8Hours10>9)
                            {
                                _cs.u8Hours10=0;
                            }
                            LCDDigit1Set(_cs.u8Hours10);
                        }
                        if (_cs.u8Hours1>3 && _cs.u8Hours10>=2)
                        {
                            _cs.u8Hours1=0;
                            _cs.u8Hours10=0;
                            LCDDigit1Set(_cs.u8Hours10);
                        }
                        LCDDigit2Set(_cs.u8Hours1);
                    }
                }
            }
            else
            {
                if (LCD_DATA_COL)
                {
                    LCD_DATA_COL=0;
                }
                else
                {
                    LCD_DATA_COL=1;
                    if (++_cs.u8Secs>59)
    //                if (++_cs.u8Secs>0) // For testing
                    {
                        _cs.u8Secs=0;
                        if (++_cs.u8Mins1>9)
                        {
                            _cs.u8Mins1=0;
                            if (++_cs.u8Mins10>5)
                            {
                                _cs.u8Mins10=0;

                                if (++_cs.u8Hours1>9)
                                {
                                    _cs.u8Hours1=0;
                                    if (++_cs.u8Hours10>9)
                                    {
                                        _cs.u8Hours10=0;
                                    }
                                    LCDDigit1Set(_cs.u8Hours10);
                                }
                                if (_cs.u8Hours1>3 && _cs.u8Hours10>=2)
                                {
                                    _cs.u8Hours1=0;
                                    _cs.u8Hours10=0;
                                    LCDDigit1Set(_cs.u8Hours10);
                                }
                                LCDDigit2Set(_cs.u8Hours1);
                            }
                            LCDDigit3Set(_cs.u8Mins10);
                        }
                        LCDDigit4Set(_cs.u8Mins1);
                    }
                }
            }
        }

        if (INTCONbits.IOCIF)
        {
            IOCBFbits.IOCBF5=0;
            INTCONbits.IOCIF=0;
            INTCONbits.IOCIE=0; // Disable IOC until the debounce timer finishes
            bDebounce=true;
            
            _cs.u8Secs=0;
            LCD_DATA_COL=1;
            
            bDebounce=true;
            TMR1=0xFFF0; // debounce timer
        }
    }
    return 0;
}
